@@ -24,10 +24,21 @@
 #include "pgpopts.h"
 #include "argvfactory.h"
 #include "defaultkey.h"
+#include "mygetopt.h"
 
 
 
 
+
+/*****************************************************************************
+ *
+ * Function : pgpopts_init
+ *
+ * Purpose  : Initialize a Pgpopts structure for future usage.
+ *
+ * Input    : opts - A pointer to an unitialized Pgpopts structure to initialize.
+ *
+ *****************************************************************************/
 void pgpopts_init(Pgpopts *opts)
 {
     assert(opts != NULL);
@@ -49,8 +60,6 @@ void pgpopts_init(Pgpopts *opts)
 
 
 
-
-
 enum
 {
     PGP_LONG_OPT_IGNORE	= -1,
@@ -70,12 +79,15 @@ enum
     PGP_LONG_OPT_CERTDEPTH,		/* cert_depth */
     PGP_LONG_OPT_COMMENT,		/* comment */
     PGP_LONG_OPT_FORCE,			/* force */
+    PGP_LONG_OPT_PUBRING,		/* pubring */
+    PGP_LONG_OPT_SECRING,		/* secring */
     
     PGP_LONG_OPT_COUNT
 };
 
 
 
+
 static struct { char *keyword; int minlen; int	index; } mapping[] =
 {
     { "armor",		5, PGP_LONG_OPT_ARMOR },
@@ -99,9 +111,9 @@ static struct { char *keyword; int minlen; int	index; } mapping[] =
     { "myname",		2, PGP_LONG_OPT_LOCALID },
     { "nomanual",	1, PGP_LONG_OPT_IGNORE },
     { "pager",		2, PGP_LONG_OPT_IGNORE },
-    { "pubring",	2, PGP_LONG_OPT_NOSUPPORT },
+    { "pubring",	2, PGP_LONG_OPT_PUBRING },
     { "randseed",	1, PGP_LONG_OPT_IGNORE },
-    { "secring",	2, PGP_LONG_OPT_NOSUPPORT },
+    { "secring",	2, PGP_LONG_OPT_SECRING },
     { "showpass",	2, PGP_LONG_OPT_IGNORE },
     { "textmode",	2, PGP_LONG_OPT_TEXTMODE },
     { "tmp",		2, PGP_LONG_OPT_IGNORE },
@@ -113,6 +125,21 @@ static struct { char *keyword; int minlen; int	index; } mapping[] =
 
 
 
+
+/*****************************************************************************
+ *
+ * Function : fetch_bool
+ *
+ * Purpose  : Determines the boolean value from an long option.
+ *
+ * Input    : keyword - The keyword to which the argument `value' was given.
+ *            value   - The value of the keyword. Either Null, "true" or "false".
+ *
+ * Output   : Returns 0 for a false boolean and 1 for a true boolean value.
+ *
+ * Errors   : Returns -1 if an error occoured.
+ *
+ *****************************************************************************/
 static int fetch_bool(char *keyword, char *value)
 {
     assert(keyword);
@@ -279,6 +306,24 @@ static int parse_longopt(Pgpopts *opts, const char *s)
             opts->comment = value;
             break;
         
+        case PGP_LONG_OPT_SECRING:
+            if (!value)
+            {
+                fprintf(stderr, "Error: Long option `secring' requires an argument.\n");
+                return -1;
+            }
+            opts->secring = value;
+            break;
+        
+        case PGP_LONG_OPT_PUBRING:
+            if (!value)
+            {
+                fprintf(stderr, "Error: Long option `pubring' requires an argument.\n");
+                return -1;
+            }
+            opts->pubring = value;
+            break;
+        
         case PGP_LONG_OPT_CHARSET:
             if (!value)
             {
@@ -322,6 +367,20 @@ static int parse_longopt(Pgpopts *opts, const char *s)
 
 
 
+
+/*****************************************************************************
+ *
+ * Function : add_argument
+ *
+ * Purpose  : Adds an argument to a Pgpopts structure. Arguments are files.
+ *
+ * Input    : pgpopts - The Pgpopts to add the argument `arg' to.
+ *            arg     - The string to add as argument to the `pgpopts'.
+ *
+ * Notes    : Don't reuse or free `arg' after calling add_argument()!! It will
+ *            not duplicated!!
+ *
+ *****************************************************************************/
 static void add_argument(Pgpopts *pgpopts, char *arg)
 {
     assert(pgpopts != NULL);
@@ -343,17 +402,21 @@ static void add_argument(Pgpopts *pgpopts, char *arg)
 
 
 
-#define OPT_INITIAL		"-hkesabcdpo:@:z:u:fwmt"
-#define OPT_KEY			"-gafxcresdvu:"
-#define OPT_ENCRYPT		"-astfo:@:z:u:wm"
-#define OPT_SIGN		"-aetfbo:z:u:"
-#define OPT_SIGN_ENCRYPT	"-atfo:@:z:u:wm"
-#define OPT_SYMMETRIC		"-atfo:z:u:wm"
-#define OPT_DECRYPT		"-pdfo:@:z:"
-
-
-
 
+/*****************************************************************************
+ *
+ * Function : check_opts_used
+ *
+ * Purpose  : Runs some consistence checks on the Pgpopts structure to 
+ *            determine if all given command line options are correctly
+ *            translated to GnuPG options.
+ *
+ * Input    : pgpopts - A Pgpopts structure to check.
+ *
+ * Output   : Returns 0 if no inconsistence are found, otherwise some 
+ *            different value.
+ *
+ *****************************************************************************/
 static int check_opts_used(Pgpopts *pgpopts)
 {
     int err = 0;
@@ -589,7 +652,46 @@ static void build_passfd(ArgvFactory *af, Pgpopts *opts)
 }
 
 
-/* Build gpg arguments from pgp options.
+#if 0
+static void build_passfd_pgpbug(ArgvFactory *af, Pgpopts *opts)
+{
+    /* Work arround a PGP 2.6x Bug */
+    if (opts->passphrase_pipe[0] == -1  &&  getenv("PGPPASSFD"))
+    {
+        argv_factory_add(af, "--passphrase-fd");
+        argv_factory_add(af, "0");
+    }
+}
+#endif
+
+
+
+static void build_keyrings(ArgvFactory *af, Pgpopts *opts)
+{
+#if 0
+    if (opts->pubring || opts->secring)
+        argv_factory_add(af, "--no-default-keyring");
+#endif
+    
+    if (opts->pubring)
+    {
+        argv_factory_add(af, "--no-default-keyring");
+        argv_factory_add(af, "--keyring");
+        argv_factory_add(af, opts->pubring);
+    }
+    
+    if (opts->secring)
+    {
+        argv_factory_add(af, "--secret-keyring");
+        argv_factory_add(af, opts->secring);
+    }
+}
+
+
+
+
+/*
+ * Build gpg arguments from pgp options.
  *
  * Please note, PGP contains a lot of bugs in the command line option
  * parsing. However, many tools and scripts which use PGP assume this
@@ -836,6 +938,8 @@ char ** pgpopts_build_argv(Pgpopts *pgpopts)
             opts.d = 0;	/* FIXME: How to handle these flags?? */
             opts.p = 0;
             build_certdepth(&af, &opts);
+            build_passfd(&af, &opts);
+            build_keyrings(&af, &opts);
             
             if (opts.outputfile)
             {
@@ -873,6 +977,7 @@ char ** pgpopts_build_argv(Pgpopts *pgpopts)
         build_compress(&af, &opts);
         build_charset(&af, &opts);
         build_passfd(&af, &opts);
+        build_keyrings(&af, &opts);
         
         if (opts.c==1 && opts.e==0 && opts.s==0)	/* encrypt symmetric */
         {
@@ -997,6 +1102,32 @@ char ** pgpopts_build_argv(Pgpopts *pgpopts)
 
 
 
+
+#define OPT_INITIAL             "hkesabcdpo:@:z:u:fwmt"
+#define OPT_KEY                 "gafxcresdvu:"
+#define OPT_ENCRYPT             "astfo:@:z:u:wm"
+#define OPT_SIGN                "aetfbo:z:u:wm"
+#define OPT_SIGN_ENCRYPT        "atfo:@:z:u:wm"
+#define OPT_SYMMETRIC           "atfo:z:u:wm"
+#define OPT_DECRYPT             "pdfo:@:z:"
+
+
+
+
+/*****************************************************************************
+ *
+ * Function : pgpopts_parse
+ *
+ * Purpose  : Parses a command line options style argv vector with PGP 2.6 like
+ *            options and fills the Pgpopts structure.
+ *
+ * Input    : pgpopts - The Pgpopts structure to fill.
+ *            argc    - The count of valid entries in the `argv' vector.
+ *            argv    - The vector with the command line options.
+ *
+ * Output   : Returns 0 if all gone right.
+ *
+ *****************************************************************************/
 int pgpopts_parse(Pgpopts *pgpopts, int argc, char *argv[])
 {
     int result;
@@ -1008,18 +1139,17 @@ int pgpopts_parse(Pgpopts *pgpopts, int argc, char *argv[])
     
     /* initial format */
     opt_format = OPT_INITIAL;
-    opterr = 0;
-    while ( (result=getopt(argc, argv, opt_format)) != EOF )
+    while ( (result=mygetopt(argc, argv, opt_format)) != EOF )
         switch(result)
         {
             case 1:
-                if (*optarg == '+')
+                if (*myoptarg == '+')
                 {
-                    if (parse_longopt(pgpopts, optarg+1))
+                    if (parse_longopt(pgpopts, myoptarg+1))
                         return -1;
                 }
                 else
-                    add_argument(pgpopts, optarg);
+                    add_argument(pgpopts, myoptarg);
                 break;
             
             case 'k':
@@ -1057,11 +1187,11 @@ int pgpopts_parse(Pgpopts *pgpopts, int argc, char *argv[])
             
             case 'z':
                 if (!pgpopts->passphrase)	/* yes, PGP uses the first -z option */
-                    pgpopts->passphrase = optarg;
+                    pgpopts->passphrase = myoptarg;
                 break;
             
-            case 'o': pgpopts->outputfile = optarg; break;
-            case 'u': pgpopts->localuserid = optarg; break;
+            case 'o': pgpopts->outputfile = myoptarg; break;
+            case 'u': pgpopts->localuserid = myoptarg; break;
             
             case 'g': pgpopts->g++; break;
             case 'a': pgpopts->a++; break;
@@ -1079,7 +1209,7 @@ int pgpopts_parse(Pgpopts *pgpopts, int argc, char *argv[])
                 return -1;
             
             case '?':
-                printf("Invalid option: -%c\n", optopt);
+                printf("Invalid option: -%c\n", myoptopt);
                 return -1;
                 break;
             
